@@ -46,7 +46,8 @@ class DefaultNames:
     HomeDirectory = ".unfurl_home"
     JobsLog = "jobs.tsv"
     ProjectDirectory = ".unfurl"
-    LocalConfigTemplate = "unfurl-local-template.yaml"
+    LocalConfigTemplate = ".unfurl-local-template.yaml"
+    InputsTemplate = "inputs-template.yaml"
 
     def __init__(self, **names):
         self.__dict__.update({name: value for name, value in names.items() if value})
