@@ -673,6 +673,17 @@ def plan(ctx, ensemble=None, **options):
     type=click.Path(exists=True),
     help="Create the ensemble in an repository outside the project.",
 )
+@click.option(
+    "--overwrite",
+    type=click.Path(exists=False),
+    help="Create ensemble in the given directory even if it exists.",
+)
+@click.option(
+    "--render",
+    is_flag=True,
+    default=False,
+    help="Generate files only (don't commit them).",
+)
 def init(ctx, projectdir, ensemble_name=None, **options):
     """
     Create a new project or, if [project_dir] exists or is inside a project, create a new ensemble.
@@ -694,6 +705,7 @@ def init(ctx, projectdir, ensemble_name=None, **options):
         if len(os.path.abspath(projectPath)) > len(os.path.abspath(projectdir)):
             projectdir = projectPath
         # this will clone the default ensemble if it exists or use ensemble-template
+        options["want_init"] = True
         message = initmod.clone(projectPath, projectdir, ensemble_name, **options)
         click.echo(message)
         return
@@ -814,12 +826,22 @@ def runtime(ctx, project_folder, init=False, **options):
 @click.option(
     "--empty", default=False, is_flag=True, help="Don't create a default ensemble."
 )
-# XXX
-# @click.option(
-#     "--use-environment",
-#     default=None,
-#     help="Associate the given environment with the ensemble.",
-# )
+@click.option(
+    "--use-environment",
+    default=None,
+    help="Associate the given environment with the ensemble.",
+)
+@click.option(
+    "--skeleton",
+    type=click.Path(exists=False),
+    help="Path to a directory of project skeleton templates.",
+)
+@click.option(
+    "--overwrite",
+    default=False,
+    is_flag=True,
+    help="Create ensemble in the given directory even if it exists.",
+)
 def clone(ctx, source, dest, **options):
     """Create a new ensemble or project from a service template or an existing ensemble or project.
 
@@ -937,7 +959,7 @@ def commit(ctx, project_or_ensemble_path, message, skip_add, no_edit, **options)
     click.echo(f"committed to {committed} repositories")
 
 
-@cli.command()
+@cli.command(short_help="Show the git status for paths relevant to this ensemble.")
 @click.pass_context
 @click.argument("project_or_ensemble_path", default=".", type=click.Path(exists=True))
 @click.option(
@@ -963,32 +985,44 @@ def git_status(ctx, project_or_ensemble_path, dirty, **options):
         click.echo(statuses)
 
 
-def _dump_manifest(manifest):
-    from .to_json import to_graphql
-
-    jsonSummary = to_graphql(manifest)
-    return json.dumps(jsonSummary, indent=2)
-
-
-@cli.command(short_help="Show the git status for paths relevant to this ensemble.")
+@cli.command()
 @click.pass_context
-@click.argument("ensemble", default=".", type=click.Path(exists=True))
+@click.argument("project_or_ensemble_path", default=".", type=click.Path(exists=False))
 @click.option(
-    "--json",
-    default=False,
-    is_flag=True,
-    help="Print ensemble in a simplified json format",
+    "--format",
+    default="deployment",
+    type=click.Choice(["blueprint", "environments", "deployment"]),
 )
-def status(ctx, ensemble, json, **options):
-    """Show the status of deployed resources in the given ensemble."""
+def export(ctx, project_or_ensemble_path, format, **options):
+    """Export ensemble in a simplified json format."""
+    from . import to_json
+
+    options.update(ctx.obj)
+    localEnv = LocalEnv(project_or_ensemble_path, options.get("home"))
+    exporter = getattr(to_json, "to_" + format)
+    jsonSummary = exporter(localEnv)
+    click.echo(json.dumps(jsonSummary, indent=2))
+
+
+@cli.command()
+@click.pass_context
+@click.argument("ensemble", default=".", type=click.Path(exists=False))
+@click.option("--query", help="Run the given expression")
+@click.option("--trace", default=0, help="Set the query's trace level")
+def status(ctx, ensemble, **options):
+    """"Show the status of deployed resources in the given ensemble."""
     options.update(ctx.obj)
     localEnv = LocalEnv(ensemble, options.get("home"))
-    # set skip_validation because we want to be able to dump incomplete service templates
-    manifest = localEnv.get_manifest(skip_validation=json)
-    if json:
-        click.echo(_dump_manifest(manifest))
-    else:
-        click.echo(manifest.status_summary())
+    manifest = localEnv.get_manifest()
+    click.echo(manifest.status_summary())
+    query = options.get("query")
+    if query:
+        from .eval import eval_for_func, RefContext
+
+        trace = options.get("trace")
+        result = eval_for_func(query, RefContext(manifest.rootResource, trace=trace))
+        click.echo("query: " + query)
+        click.echo(result)
 
 
 @cli.command()
