@@ -4,6 +4,7 @@ from ..configurator import Configurator
 from ..result import ResultsMap
 from ..util import register_short_names
 from ..support import Status
+from ..planrequests import set_default_command
 import importlib
 
 # need to define these now because these configurators are lazily imported
@@ -16,6 +17,12 @@ register_short_names(
 )
 
 
+class CmdConfigurator(Configurator):
+    @classmethod
+    def set_config_spec_args(klass, kw: dict, target):
+        return set_default_command(kw, "")
+
+
 class PythonPackageCheckConfigurator(Configurator):
     def run(self, task):
         try:
