@@ -1,28 +1,30 @@
 # Copyright (c) 2020 Adam Souzis
 # SPDX-License-Identifier: MIT
-from __future__ import absolute_import
-import sys
+
 import collections
+from collections.abc import MutableSequence
 import functools
 import logging
-from .. import getLogLevel
-from ..util import assertForm, saveToTempfile
-from ..configurator import Status
-from ..result import serializeValue
-from . import TemplateConfigurator
+import sys
+
 import ansible.constants as C
 from ansible import context
 from ansible.cli.playbook import PlaybookCLI
-from ansible.plugins.callback.default import CallbackModule
 from ansible.module_utils import six
+from ansible.plugins.callback.default import CallbackModule
 from ansible.utils.display import Display
 
+from . import TemplateConfigurator
+from ..configurator import Status
+from ..result import serialize_value
+from ..util import assert_form
+
 display = Display()
 
 logger = logging.getLogger("unfurl")
 
 
-def getAnsibleResults(result, extraKeys=(), facts=()):
+def get_ansible_results(result, extraKeys=(), facts=()):
     """
     Returns a dictionary containing at least:
 
@@ -65,10 +67,11 @@ def getAnsibleResults(result, extraKeys=(), facts=()):
     outputs = {}
     if facts:
         ansible_facts = result._result.get("ansible_facts")
-        if ansible_facts:
-            for fact in facts:
-                if fact in ansible_facts:
-                    outputs[fact] = ansible_facts[fact]
+        for fact in facts:
+            if ansible_facts and fact in ansible_facts:
+                outputs[fact] = ansible_facts[fact]
+            elif fact in result._result:
+                outputs[fact] = result._result[fact]
     resultDict["ignored"] = ignoredKeys
     return resultDict, outputs
 
@@ -77,13 +80,13 @@ class AnsibleConfigurator(TemplateConfigurator):
     """The current resource is the inventory."""
 
     def __init__(self, configSpec):
-        super(AnsibleConfigurator, self).__init__(configSpec)
+        super().__init__(configSpec)
         self._cleanupRoutines = []
 
-    def canDryRun(self, task):
+    def can_dry_run(self, task):
         return True
 
-    def _makeInventoryFromGroup(self, group, includeInstances):
+    def _make_inventory_from_group(self, group, includeInstances):
         hosts = {}
         vars = {}
         children = {}
@@ -91,17 +94,19 @@ class AnsibleConfigurator(TemplateConfigurator):
         #  if includeInstances:
         #     for member in group.memberInstances:
         #         hosts[member.name] = self._getHostVars(member)
-        for child in group.memberGroups:
-            if child.isCompatibleType("unfurl.groups.AnsibleInventoryGroup"):
-                children[child] = self._makeInventoryFromGroup(child, includeInstances)
+        for child in group.member_groups:
+            if child.is_compatible_type("unfurl.groups.AnsibleInventoryGroup"):
+                children[child] = self._make_inventory_from_group(
+                    child, includeInstances
+                )
         vars.update(group.properties.get("hostvars", {}))
         return dict(hosts=hosts, vars=vars, children=children)
 
-    def _getHostVars(self, node):
+    def _get_host_vars(self, node):
         # return ansible_connection, ansible_host, ansible_user, ansible_port
-        connections = node.getCapabilities("endpoint")
+        connections = node.get_capabilities("endpoint")
         for connection in connections:
-            if connection.template.isCompatibleType(
+            if connection.template.is_compatible_type(
                 "unfurl.capabilities.Endpoint.Ansible"
             ):
                 break
@@ -119,7 +124,7 @@ class AnsibleConfigurator(TemplateConfigurator):
             hostVars["ansible_host"] = hostVars["ip_address"]
         return hostVars
 
-    def _updateVars(self, connection, hostVars):
+    def _update_vars(self, connection, hostVars):
         creds = connection.attributes.get("credential")
         if creds:
             if "user" in creds:
@@ -131,14 +136,14 @@ class AnsibleConfigurator(TemplateConfigurator):
                 hostVars.update(creds["keys"])
         hostVars.update(connection.attributes.get("hostvars", {}))
 
-    def _makeInventory(self, host, allVars, task):
+    def _make_inventory(self, host, allVars, task):
         if host:
-            hostVars = self._getHostVars(host)
-            connection = task.findConnection(
+            hostVars = self._get_host_vars(host)
+            connection = task.find_connection(
                 host, "unfurl.relationships.ConnectsTo.Ansible"
             )
             if connection:
-                self._updateVars(connection, hostVars)
+                self._update_vars(connection, hostVars)
 
             if "ansible_host" not in hostVars:
                 ip_address = host.attributes.get("public_ip") or host.attributes.get(
@@ -150,15 +155,15 @@ class AnsibleConfigurator(TemplateConfigurator):
             project_id = host.attributes.get("project_id")
             if project_id:
                 # hack for gcp because this name is set in .ssh/config by `gcloud compute config-ssh --quiet`
-                hostname = "%s.%s.%s" % (host.name, host.attributes["zone"], project_id)
+                hostname = f"{host.name}.{host.attributes['zone']}.{project_id}"
             else:
                 hostname = host.name
             hosts = {hostname: hostVars}
 
             children = {
-                group.name: self._makeInventoryFromGroup(group, False)
+                group.name: self._make_inventory_from_group(group, False)
                 for group in host.template.groups
-                if group.isCompatibleType("unfurl.groups.AnsibleInventoryGroup")
+                if group.is_compatible_type("unfurl.groups.AnsibleInventoryGroup")
             }
         else:
             hostVars = {}
@@ -176,7 +181,7 @@ class AnsibleConfigurator(TemplateConfigurator):
         # note: allVars is inventory vars shared by all hosts
         return dict(all=dict(hosts=hosts, vars=allVars, children=children))
 
-    def getInventory(self, task):
+    def get_inventory(self, task, cwd):
         inventory = task.inputs.get("inventory")
         if inventory and isinstance(inventory, six.string_types):
             # XXX if user set inventory file we can create a folder to merge them
@@ -186,9 +191,9 @@ class AnsibleConfigurator(TemplateConfigurator):
         if not inventory:
             # XXX merge inventory
             # default to localhost if not inventory
-            inventory = self._makeInventory(task.operationHost, inventory or {}, task)
+            inventory = self._make_inventory(task.operation_host, inventory or {}, task)
         # XXX cache and reuse file
-        return saveToTempfile(inventory, "-inventory.yaml").name
+        return cwd.write_file(inventory, "inventory.yaml")
         # don't worry about the warnings in log, see:
         # https://github.com/ansible/ansible/issues/33132#issuecomment-346575458
         # https://github.com/ansible/ansible/issues/33132#issuecomment-363908285
@@ -203,15 +208,15 @@ class AnsibleConfigurator(TemplateConfigurator):
                 pass
         self._cleanupRoutines = []
 
-    def getVars(self, task):
+    def get_vars(self, task):
         vars = task.inputs.context.vars.copy()
         vars["__unfurl"] = task.inputs.context
         return vars
 
-    def _makePlayBook(self, playbook, task):
-        assertForm(playbook, collections.MutableSequence)
+    def _make_playbook(self, playbook, task):
+        assert_form(playbook, MutableSequence)
         # XXX use host group instead of localhost depending on operation_host
-        hosts = task.operationHost and task.operationHost.name or "localhost"
+        hosts = task.operation_host and task.operation_host.name or "localhost"
         if playbook and not "hosts" in playbook[0]:
             play = dict(hosts=hosts, gather_facts=False, tasks=playbook)
             if hosts == "localhost":
@@ -220,58 +225,62 @@ class AnsibleConfigurator(TemplateConfigurator):
         else:
             return playbook
 
-    def findPlaybook(self, task):
+    def find_playbook(self, task):
         return task.inputs["playbook"]
 
-    def getPlaybook(self, task):
-        playbook = self.findPlaybook(task)
+    def get_playbook(self, task, cwd):
+        playbook = self.find_playbook(task)
         if isinstance(playbook, six.string_types):
             # assume it's file path
             return playbook
-        playbook = self._makePlayBook(playbook, task)
-        envvars = task.getEnvironment(True)
+        playbook = self._make_playbook(playbook, task)
+        envvars = task.get_environment(True)
         for play in playbook:
             play["environment"] = envvars
-        return saveToTempfile(serializeValue(playbook), "-playbook.yml").name
+        return cwd.write_file(serialize_value(playbook), "playbook.yml")
 
-    def getPlaybookArgs(self, task):
+    def get_playbook_args(self, task):
         args = task.inputs.get("playbookArgs", [])
-        if not isinstance(args, collections.MutableSequence):
+        if not isinstance(args, MutableSequence):
             args = [args]
-        if task.dryRun:
+        if task.dry_run:
             args.append("--check")
         if task.configSpec.timeout:
-            args.append("--timeout=%s" % task.configSpec.timeout)
+            args.append(f"--timeout={task.configSpec.timeout}")
         if task.verbose > 0:
             args.append("-" + ("v" * task.verbose))
         return list(args)
 
-    def getResultKeys(self, task, results):
+    def get_result_keys(self, task, results):
         return task.inputs.get("resultKeys", [])
 
-    def processResult(self, task, result):
-        self.processResultTemplate(
+    def process_result(self, task, result):
+        result.success = not self.process_result_template(
             task, dict(result.result, success=result.success, outputs=result.outputs)
         )
         return result
 
+    def render(self, task):
+        cwd = task.set_work_folder()
+        # build host inventory from resource
+        inventory = self.get_inventory(task, cwd)
+        playbook = self.get_playbook(task, cwd)
+        playbookArgs = self.get_playbook_args(task)
+        args = _render_playbook(playbook, inventory, playbookArgs)
+        return args
+
     def run(self, task):
         try:
-            # build host inventory from resource
-            inventory = self.getInventory(task)
-            playbook = self.getPlaybook(task)
-
+            args = task.rendered
             # build vars from inputs
-            extraVars = self.getVars(task)
-            if task.operationHost and task.operationHost.templar:
-                vault_secrets = task.operationHost.templar._loader._vault.secrets
+            extraVars = self.get_vars(task)
+            if task.operation_host and task.operation_host.templar:
+                vault_secrets = task.operation_host.templar._loader._vault.secrets
             else:
                 vault_secrets = None
-            resultCallback = runPlaybooks(
-                [playbook],
-                inventory,
+            resultCallback = _run_playbooks(
+                args,
                 extraVars,
-                self.getPlaybookArgs(task),
                 vault_secrets,
             )
 
@@ -290,12 +299,14 @@ class AnsibleConfigurator(TemplateConfigurator):
             )
 
             if resultCallback.results:
-                resultKeys = self.getResultKeys(task, resultCallback.results)
+                resultKeys = self.get_result_keys(task, resultCallback.results)
                 factKeys = list(task.configSpec.outputs)
                 # each task in a playbook will have a corresponding result
                 resultList, outputList = zip(
                     *map(
-                        lambda result: getAnsibleResults(result, resultKeys, factKeys),
+                        lambda result: get_ansible_results(
+                            result, resultKeys, factKeys
+                        ),
                         resultCallback.results,
                     )
                 )
@@ -315,10 +326,12 @@ class AnsibleConfigurator(TemplateConfigurator):
             else:
                 modified = False
 
-            result = task.done(
-                status == Status.ok and not task._errors,
-                modified,
-                targetStatus,
+            success = status == Status.ok and not task._errors
+            result = self.done(
+                task,
+                success=success,
+                status=targetStatus,
+                modified=modified,
                 result=results,
                 outputs=outputs,
             )
@@ -330,7 +343,7 @@ class AnsibleConfigurator(TemplateConfigurator):
                 # this can update resources so don't do it on error
                 # XXX align with shell configurator behavior
                 # (which is called unconditionally and before done())
-                result = self.processResult(task, result)
+                result = self.process_result(task, result)
             yield result
 
         finally:
@@ -349,7 +362,7 @@ class ResultCallback(CallbackModule):
     # see ansible.executor.task_result.TaskResult and ansible.playbook.task.Task
 
     def __init__(self):
-        super(ResultCallback, self).__init__()
+        super().__init__()
         self.results = []
         # named tuple of OrderedDict<task_name:list<result>>
         self.resultsByStatus = _ResultsByStatus(
@@ -361,14 +374,14 @@ class ResultCallback(CallbackModule):
     def get_option(self, k):
         return False
 
-    def getInfo(self, result):
+    def get_info(self, result):
         host = result._host
         taskname = result.task_name
         fields = result._task_fields.keys()
         keys = result._result.keys()
-        return "%s: %s(%s) => %s" % (host, taskname, fields, keys)
+        return f"{host}: {taskname}({fields}) => {keys}"
 
-    def _addResult(self, status, result):
+    def _add_result(self, status, result):
         self.results.append(result)
         if result._result.get("changed", False):
             self.changed += 1
@@ -378,24 +391,24 @@ class ResultCallback(CallbackModule):
         )
 
     def v2_runner_on_ok(self, result):
-        self._addResult("ok", result)
+        self._add_result("ok", result)
         # print("ok", self.getInfo(result))
-        super(ResultCallback, self).v2_runner_on_ok(result)
+        super().v2_runner_on_ok(result)
 
     def v2_runner_on_skipped(self, result):
-        self._addResult("skipped", result)
+        self._add_result("skipped", result)
         # print("skipped", self.getInfo(result))
-        super(ResultCallback, self).v2_runner_on_skipped(result)
+        super().v2_runner_on_skipped(result)
 
     def v2_runner_on_failed(self, result, **kwargs):
-        self._addResult("failed", result)
+        self._add_result("failed", result)
         # print("failed", self.getInfo(result))
-        super(ResultCallback, self).v2_runner_on_failed(result, **kwargs)
+        super().v2_runner_on_failed(result, **kwargs)
 
     def v2_runner_on_unreachable(self, result):
-        self._addResult("unreachable", result)
+        self._add_result("unreachable", result)
         # print("unreachable", self.getInfo(result))
-        super(ResultCallback, self).v2_runner_on_unreachable(result)
+        super().v2_runner_on_unreachable(result)
 
 
 def _init_global_context(options):
@@ -405,9 +418,18 @@ def _init_global_context(options):
     context.CLIARGS._store = vars(options)
 
 
-def runPlaybooks(playbooks, _inventory, params=None, args=None, vault_secrets=None):
+def run_playbooks(playbook, _inventory, params=None, args=None):
+    args = _render_playbook(playbook, _inventory, args)
+    return _run_playbooks(args, params)
+
+
+def _render_playbook(playbook, _inventory, args):
     inventoryArgs = ["-i", _inventory] if _inventory else []
-    args = ["ansible-playbook"] + inventoryArgs + (args or []) + playbooks
+    args = ["ansible-playbook"] + inventoryArgs + (args or []) + [playbook]
+    return args
+
+
+def _run_playbooks(args, params=None, vault_secrets=None):
     logger.info("running " + " ".join(args))
     cli = PlaybookCLI(args)
 
@@ -437,7 +459,7 @@ def runPlaybooks(playbooks, _inventory, params=None, args=None, vault_secrets=No
     cli._play_prereqs = hook_play_prereqs
 
     oldVerbosity = display.verbosity
-    if getLogLevel() <= 10:  # debug
+    if logger.getEffectiveLevel() <= logging.DEBUG:
         display.verbosity = 2
     try:
         resultsCB.exit_code = cli.run()
