@@ -1,7 +1,7 @@
 # Copyright (c) 2020 Adam Souzis
 # SPDX-License-Identifier: MIT
 """
-enviroment:
+environment:
 timeout:
 inputs:
  command: "--switch {{ '.::foo' | ref }}"
@@ -28,6 +28,7 @@ import os
 import sys
 import shlex
 import six
+import re
 from click.termui import unstyle
 
 if os.name == "posix" and sys.version_info[0] < 3:
@@ -81,15 +82,21 @@ def _run(*args, **kwargs):
 
 
 def _truncate(s):
+    if not s:
+        return ""
     if len(s) > 1000:
-        return "%s [%s omitted...]  %s" % (s[:500], len(s), s[-500:])
+        return f"{s[:500]} [{len(s)} omitted...]  {s[-500:]}"
     return s
 
 
+def clean_output(value: str) -> str:
+    return re.sub(r"[\x00-\x08\x0e-\x1f\x7f-\x9f]", "", unstyle(value))
+
+
 # XXX we should know if cmd if not os.access(implementation, os.X):
 class ShellConfigurator(TemplateConfigurator):
-    excludeFromDigest = TemplateConfigurator.excludeFromDigest + ("cwd", "echo")
-    _defaultCmd = None
+    exclude_from_digest = TemplateConfigurator.exclude_from_digest + ("cwd", "echo")
+    _default_cmd = None
 
     @staticmethod
     def _cmd(cmd, keeplines):
@@ -102,7 +109,7 @@ class ShellConfigurator(TemplateConfigurator):
             cmd = shlex.split(cmd)
         return cmdStr, cmd
 
-    def runProcess(
+    def run_process(
         self,
         cmd,
         shell=False,
@@ -175,7 +182,7 @@ class ShellConfigurator(TemplateConfigurator):
             err.error = err
             return err
 
-    def _handleResult(self, task, result, cwd, successCodes=(0,)):
+    def _handle_result(self, task, result, cwd, successCodes=(0,)):
         error = result.error or result.returncode not in successCodes or result.timeout
         if error:
             task.logger.warning('shell task run failure: "%s" in %s', result.cmd, cwd)
@@ -191,39 +198,34 @@ class ShellConfigurator(TemplateConfigurator):
             task.logger.info("shell task run success: %s", result.cmd)
             task.logger.debug("shell task output: %s", _truncate(result.stdout))
         # strips terminal escapes after we printed output via logger
-        result.stdout = unstyle(result.stdout or "")
-        result.stderr = unstyle(result.stderr or "")
+        result.stdout = clean_output(result.stdout or "")
+        result.stderr = clean_output(result.stderr or "")
         return not error
 
-    def _processResult(self, task, result, cwd):
-        success = self._handleResult(task, result, cwd)
+    def _process_result(self, task, result, cwd):
+        success = self._handle_result(task, result, cwd)
         resultDict = result.__dict__.copy()
         resultDict["success"] = success
-        self.processResultTemplate(task, resultDict)
+        self.process_result_template(task, resultDict)
         if task._errors:
             return False
         return success
 
-    def canRun(self, task):
+    def can_run(self, task):
         params = task.inputs
-        cmd = params.get("command", self._defaultCmd)
+        cmd = params.get("command", self._default_cmd)
         if not cmd:
             return "missing command to execute"
         if isinstance(cmd, list) and not params.get("shell") and not which(cmd[0]):
-            return "'%s' is not executable" % cmd[0]
+            return f"'{cmd[0]}' is not executable"
         return True
 
-    def canDryRun(self, task):
+    def can_dry_run(self, task):
         return task.inputs.get("dryrun")
 
-    def run(self, task):
+    def render(self, task):
         params = task.inputs
         cmd = params["command"]
-        isString = isinstance(cmd, six.string_types)
-        # default for shell: True if command is a string otherwise False
-        shell = params.get("shell", isString)
-        env = task.getEnvironment(False)
-
         cwd = params.get("cwd")
         if cwd:
             # if cwd is relative, make it relative to task.cwd
@@ -231,11 +233,21 @@ class ShellConfigurator(TemplateConfigurator):
             cwd = os.path.abspath(os.path.join(task.cwd, cwd))
         else:
             cwd = task.cwd
-        keeplines = params.get("keeplines")
-
         cmd = self.resolve_dry_run(cmd, task)
-        echo = params.get("echo",  task.verbose > -1)
-        result = self.runProcess(
+        return [cmd, cwd]
+
+    def run(self, task):
+        cmd, cwd = task.rendered
+        task.logger.trace("executing %s", cmd)
+        params = task.inputs
+        isString = isinstance(cmd, six.string_types)
+        # default for shell: True if command is a string otherwise False
+        shell = params.get("shell", isString)
+        env = task.get_environment(False)
+        task.logger.trace("shell using env %s", env)
+        keeplines = params.get("keeplines")
+        echo = params.get("echo", task.verbose > -1)
+        result = self.run_process(
             cmd,
             shell=shell,
             timeout=task.configSpec.timeout,
@@ -244,14 +256,12 @@ class ShellConfigurator(TemplateConfigurator):
             keeplines=keeplines,
             echo=echo,
         )
-        success = self._processResult(task, result, cwd)
-        done = task.inputs.get("done", {})
-        success = done.pop("success", success)
-        yield task.done(success, result=result.__dict__, **done)
+        success = self._process_result(task, result, cwd)
+        yield self.done(task, success=success, result=result.__dict__)
 
     def resolve_dry_run(self, cmd, task):
         is_string = isinstance(cmd, six.string_types)
-        if task.dryRun and isinstance(task.inputs.get("dryrun"), six.string_types):
+        if task.dry_run and isinstance(task.inputs.get("dryrun"), six.string_types):
             dry_run_arg = task.inputs["dryrun"]
             if "%dryrun%" in cmd:  # replace %dryrun%
                 if is_string:
