@@ -1,8 +1,7 @@
 # Copyright (c) 2020 Adam Souzis
 # SPDX-License-Identifier: MIT
-import os.path
-from unfurl.eval import Ref, mapValue
-from unfurl.support import abspath, getdir
+from unfurl.eval import Ref, map_value
+from unfurl.projectpaths import _abspath, _getdir
 from unfurl.util import which
 from jinja2.filters import contextfilter
 
@@ -12,19 +11,19 @@ from jinja2.filters import contextfilter
 @contextfilter
 def ref(context, ref, **vars):
     refContext = context["__unfurl"]
-    return Ref(ref, vars=vars).resolveOne(refContext)
+    return Ref(ref, vars=vars).resolve_one(refContext)
 
 
 @contextfilter
-def mapValueFilter(context, ref, **vars):
+def map_value_filter(context, ref, **vars):
     refContext = context["__unfurl"]
     if vars:
         refContext = refContext.copy(vars=vars)
-    return mapValue(ref, refContext)
+    return map_value(ref, refContext)
 
 
 @contextfilter
-def _abspath(context, path, relativeTo=None, mkdir=True):
+def abspath(context, path, relativeTo=None, mkdir=False):
     """
     {{ 'foo' | abspath }}
 
@@ -33,22 +32,26 @@ def _abspath(context, path, relativeTo=None, mkdir=True):
     {{ 'foo' | abspath('local') }}
     """
     refContext = context["__unfurl"]
-    return abspath(refContext, path, relativeTo, mkdir)
+    external = _abspath(refContext, path, relativeTo, mkdir)
+    refContext.add_external_reference(external)
+    return external.get()
 
 
 @contextfilter
-def get_dir(context, relativeTo, mkdir=True):
+def get_dir(context, relativeTo, mkdir=False):
     refContext = context["__unfurl"]
-    return getdir(refContext, relativeTo, mkdir)
+    filepath = _getdir(refContext, relativeTo, mkdir)
+    refContext.add_external_reference(filepath)
+    return filepath.get()
 
 
-class FilterModule(object):
+class FilterModule:
     def filters(self):
         return {
             "ref": ref,
             "eval": ref,
-            "mapValue": mapValueFilter,
-            "abspath": _abspath,
+            "mapValue": map_value_filter,
+            "abspath": abspath,
             "get_dir": get_dir,
             "which": which,
         }
