@@ -205,6 +205,7 @@ def render_project(
     names=DefaultNames,
     use_context=None,
     mono=False,
+    no_secrets=False,
 ):
     """
     Creates a folder named `projectdir` with a git repository with the following files:
@@ -259,20 +260,24 @@ def render_project(
         vars,
         templateDir,
     )
+
     _warn_about_new_password(localProjectConfig)
 
-    write_project_config(
-        os.path.join(projectdir, "secrets"),
-        names.SecretsConfig,
-        "secrets.yaml.j2",
-        vars,
-        templateDir,
-    )
-
     localInclude = "+?include-local: " + os.path.join("local", localConfigFilename)
-    secretsInclude = "+?include-secrets: " + os.path.join(
-        "secrets", names.SecretsConfig
-    )
+
+    if no_secrets:
+        secretsInclude = ""
+    else:
+        write_project_config(
+            os.path.join(projectdir, "secrets"),
+            names.SecretsConfig,
+            "secrets.yaml.j2",
+            vars,
+            templateDir,
+        )
+        secretsInclude = "+?include-secrets: " + os.path.join(
+            "secrets", names.SecretsConfig
+        )
 
     # note: local overrides secrets
     vars = dict(include=secretsInclude + "\n" + localInclude, vaultid=vaultid)
@@ -461,19 +466,20 @@ def create_project(
         assert homeProject
         homeProject.localConfig.register_project(newProject, create_context)
 
-    if password_vault:
-        yaml = make_yaml(password_vault)
-        commit_secrets(os.path.dirname(projectConfigPath), yaml)
+    if not kw.get("render"):
+        if password_vault:
+            yaml = make_yaml(password_vault)
+            commit_secrets(os.path.dirname(projectConfigPath), yaml)
 
-    _commit_repos(
-        projectdir,
-        repo,
-        not mono and ensembleRepo,
-        shared,
-        kw,
-        ensembleDir,
-        newHome,
-    )
+        _commit_repos(
+            projectdir,
+            repo,
+            not mono and ensembleRepo,
+            shared,
+            kw,
+            ensembleDir,
+            newHome,
+        )
 
     return newHome, projectConfigPath, repo
 
@@ -490,7 +496,7 @@ def clone_local_repos(manifest, sourceProject, targetProject):
             targetProject.find_or_clone(repo)
 
 
-def _create_ensemble_repo(manifest, repo):
+def _create_ensemble_repo(manifest, repo, commit=True):
     destDir = os.path.dirname(manifest.manifest.path)
     if not repo:
         repo = _create_repo(destDir)
@@ -501,8 +507,9 @@ def _create_ensemble_repo(manifest, repo):
     with open(manifest.manifest.path, "w") as f:
         manifest.dump(f)
 
-    repo.repo.index.add([manifest.manifest.path])
-    repo.repo.index.commit("Default ensemble repository boilerplate")
+    if commit:
+        repo.repo.index.add([manifest.manifest.path])
+        repo.repo.index.commit("Default ensemble repository boilerplate")
     return repo
 
 
@@ -566,31 +573,6 @@ def _get_ensemble_paths(sourcePath, sourceProject):
     return {}
 
 
-def _create_ensemble_from_template(templateVars, project, destDir, manifestName):
-    from unfurl import yamlmanifest
-
-    assert project
-    sourceDir = os.path.normpath(
-        os.path.join(project.projectRoot, templateVars["sourceDir"])
-    )
-    specRepo, relPath, revision, bare = project.find_path_in_repos(sourceDir)
-    if not specRepo:
-        raise UnfurlError(
-            '"%s" is not in a git repository. Cloning from plain file directories not yet supported'
-            % os.path.abspath(sourceDir)
-        )
-    manifestPath = write_ensemble_manifest(
-        os.path.join(project.projectRoot, destDir),
-        manifestName,
-        specRepo,
-        sourceDir,
-        templateVars,
-    )
-    localEnv = LocalEnv(manifestPath, project=project)
-    manifest = yamlmanifest.ReadOnlyManifest(localEnv=localEnv)
-    return localEnv, manifest
-
-
 def find_project(source, home_path):
     sourceRoot = Project.find_path(source)
     if sourceRoot:
@@ -602,7 +584,6 @@ def find_project(source, home_path):
 
 def _get_context_and_shared_repo(project, options):
     # when creating ensemble, get the default project for the given context if set
-    # XXX if not --new-repository
     shared_repo = None
     shared = options.get("shared_repository")
     context = options.get("use_environment")
@@ -610,6 +591,10 @@ def _get_context_and_shared_repo(project, options):
         context = project.get_default_context()
     if not shared and context:
         shared = project.get_default_project_path(context)
+        if not shared and context not in project.contexts:
+            raise UnfurlError(
+                f'environment "{context}" not found in project at "{project.projectRoot}"'
+            )
     if shared:
         shared_repo = Repo.find_containing_repo(shared)
         if not shared_repo:
@@ -638,6 +623,7 @@ class EnsembleBuilder:
         self.dest_path = None  # step 3 relative path in dest_project
 
         self.manifest = None  # final step
+        self.logger = logging.getLogger("unfurl")
 
     def create_project_from_ensemble(self, dest):
         # XXX create a new project from scratch for the ensemble
@@ -663,7 +649,7 @@ class EnsembleBuilder:
             self.source_project,
         )
         (self.environment, self.shared_repo) = _get_context_and_shared_repo(
-            self.source_project, self.options
+            self.dest_project, self.options
         )
 
     @staticmethod
@@ -681,6 +667,43 @@ class EnsembleBuilder:
             manifestName = DefaultNames.Ensemble
         return destDir, manifestName
 
+    def _get_inputs_template(self):
+        local_template = os.path.join(
+            self.source_project.projectRoot, DefaultNames.InputsTemplate
+        )
+        if os.path.isfile(local_template):
+            with open(local_template) as s:
+                return s.read()
+        return None
+
+    def _create_ensemble_from_template(self, project, destDir, manifestName):
+        from unfurl import yamlmanifest
+
+        specProject = self.dest_project
+        assert project
+        sourceDir = os.path.normpath(
+            os.path.join(
+                self.source_project.projectRoot, self.templateVars["sourceDir"]
+            )
+        )
+        specRepo, relPath, revision, bare = specProject.find_path_in_repos(sourceDir)
+        if not specRepo:
+            raise UnfurlError(
+                '"%s" is not in a git repository. Cloning from plain file directories not yet supported'
+                % os.path.abspath(sourceDir)
+            )
+        manifestPath = write_ensemble_manifest(
+            os.path.join(project.projectRoot, destDir),
+            manifestName,
+            specRepo,
+            sourceDir,
+            self.templateVars,
+            self.options.get("skeleton"),
+        )
+        localEnv = LocalEnv(manifestPath, project=project)
+        manifest = yamlmanifest.ReadOnlyManifest(localEnv=localEnv)
+        return localEnv, manifest
+
     def create_new_ensemble(self):
         """
         If "localEnv" is in templateVars, clone that ensemble;
@@ -702,7 +725,11 @@ class EnsembleBuilder:
         destDir, manifestName = self._get_ensemble_dir(self.dest_path)
         # choose a destDir that doesn't conflict with an existing folder
         # (i.e. if default ensemble already exists)
-        destDir = destProject.get_unique_path(destDir)
+        if self.options.get("overwrite"):
+            destDir = os.path.join(destProject.projectRoot, destDir)
+        else:
+            destDir = destProject.get_unique_path(destDir)
+
         # destDir is now absolute
         targetPath = os.path.normpath(os.path.join(destDir, manifestName))
         assert (not self.shared_repo) or targetPath.startswith(
@@ -713,13 +740,13 @@ class EnsembleBuilder:
         )
 
         templateVars = self.templateVars
-        if "localEnv" not in templateVars:
+        if "localEnv" not in templateVars or self.options.get("want_init"):
             # we found a template file to clone
-            localEnv, manifest = _create_ensemble_from_template(
-                self.templateVars, destProject, destDir, manifestName
+            templateVars["inputs"] = self._get_inputs_template()
+            localEnv, manifest = self._create_ensemble_from_template(
+                destProject, destDir, manifestName
             )
         else:
-            # didn't find a template file
             # look for an ensemble at the given path or use the source project's default
             localEnv = templateVars["localEnv"]
             manifest = yamlmanifest.clone(localEnv, targetPath)
@@ -727,6 +754,7 @@ class EnsembleBuilder:
         _create_ensemble_repo(
             manifest,
             self.shared_repo or self.mono and self.dest_project.project_repoview.repo,
+            not self.options.get("render"),
         )
 
         if destProject.projectRoot != self.dest_project.projectRoot:
@@ -742,42 +770,53 @@ class EnsembleBuilder:
         self.manifest = manifest
         return destDir
 
-    def clone_local_project(self, sourceProject, dest_dir):
-        # clone the source project's git repo
+    def clone_local_project(self, currentProject, sourceProject, dest_dir):
         self.source_path = sourceProject.get_relative_path(self.input_source)
         assert not self.source_path.startswith(
             ".."
         ), f"{self.source_path} should be inside the project"
-        newrepo = sourceProject.project_repoview.repo.clone(dest_dir)
-        search = os.path.join(
-            dest_dir, sourceProject.project_repoview.path, self.source_path
-        )
+        if currentProject:
+            newrepo = currentProject.find_or_create_working_dir(
+                sourceProject.project_repoview.repo.url,
+            )
+            search = os.path.join(newrepo.working_dir, self.source_path)
+        else:
+            # clone the source project's git repo
+            newrepo = sourceProject.project_repoview.repo.clone(dest_dir)
+            search = os.path.join(
+                dest_dir, sourceProject.project_repoview.path, self.source_path
+            )
+
         self.source_project = find_project(search, self.home_path)
         assert (
             self.source_project
         ), f"project not found in {search}, cloned to {newrepo.working_dir}"
         return self.source_project
 
-    def clone_remote_project(self, destDir):
+    def clone_remote_project(self, currentProject, destDir):
         # check if source is a git url
         repoURL, filePath, revision = split_git_url(self.input_source)
 
-        if os.path.exists(destDir) and os.listdir(destDir):
-            raise UnfurlError(
-                f'Can not clone project into "{destDir}": folder is not empty'
-            )
-
-        # #lone the remote repo to destDir
-        Repo.create_working_dir(repoURL, destDir, revision)
+        if currentProject:
+            repo = currentProject.find_or_create_working_dir(repoURL, revision)
+            destDir = repo.working_dir
+        else:
+            if os.path.exists(destDir) and os.listdir(destDir):
+                raise UnfurlError(
+                    f'Can not clone project into "{destDir}": folder is not empty'
+                )
+            # clone the remote repo to destDir
+            Repo.create_working_dir(repoURL, destDir, revision)
 
         targetDir = os.path.join(destDir, filePath)
-        sourceRoot = Project.find_path(targetDir)
-        if not sourceRoot:
+        sourceProjectRoot = Project.find_path(targetDir)
+        self.logger.debug(f'cloned {self.input_source}" to "{destDir}"')
+        if not sourceProjectRoot:
             raise UnfurlError(
                 f'Error: cloned "{self.input_source}" to "{destDir}" but couldn\'t find an Unfurl project'
             )
 
-        self.source_project = find_project(sourceRoot, self.home_path)
+        self.source_project = find_project(sourceProjectRoot, self.home_path)
         # set source to point to the cloned project
         self.source_path = self.source_project.get_relative_path(targetDir)
         return self.source_project
@@ -790,21 +829,17 @@ class EnsembleBuilder:
         if existingDestProject:
             #     set that as the dest_project
             self.dest_project = existingDestProject
-            if existingSourceProject is not existingDestProject and new_project:
-                # we cloned a new source project inside of an existing project
-                # add the cloned project's repo to the currentProject so we can find it later
-                # to set it as the ensemble's spec repository
-                existingDestProject.workingDirs[
-                    self.source_project.projectRoot
-                ] = self.source_project.project_repoview
-            # path from dest to source
         else:
             # otherwise set source_project as the dest_project
             self.dest_project = self.source_project
             if new_project:
                 # finishing creating the new project
                 # create local/unfurl.yaml in the new project
-                _create_local_config(self.source_project)
+                if _create_local_config(self.source_project, self.logger):
+                    # reload project with the new local project config
+                    self.dest_project = find_project(
+                        self.source_project.projectRoot, self.home_path
+                    )
                 # set "" as dest because we already "consumed" dest by cloning the project to that location
                 dest = ""
 
@@ -903,32 +938,39 @@ def clone(source, dest, ensemble_name=DefaultNames.EnsembleDirectory, **options)
     ### step 1: clone the source repository and set the the source path
     sourceProject = None
     isRemote = is_url_or_git_path(source)
+
     if isRemote:
-        builder.clone_remote_project(dest)
+        builder.clone_remote_project(currentProject, dest)
     else:
         sourceProject = find_project(source, builder.home_path)
-        if not sourceProject:
-            # source wasn't in a project
+        if not sourceProject or not sourceProject.project_repoview.repo:
+            # source wasn't in a project in a repo
             # XXX currently just raises error
             return builder.create_project_from_ensemble(dest)
 
         relDestDir = sourceProject.get_relative_path(dest)
-        if relDestDir.startswith(".."):
-            # dest is outside the source project, so clone the source project
-            builder.clone_local_project(sourceProject, dest)
+        if relDestDir.startswith("..") and (
+            not currentProject
+            or not currentProject.project_repoview.repo
+            or currentProject.project_repoview.repo.working_dir
+            != sourceProject.project_repoview.repo.working_dir
+        ):
+            # dest is outside the source project and is in a different repo than the current project
+            # so clone the source project
+            builder.clone_local_project(currentProject, sourceProject, dest)
         else:
-            # dest is in the source project
+            # dest is in the source project's repo
             # so don't need to clone, just need to create an ensemble
             builder.set_source(sourceProject)
 
     assert builder.source_project
 
-    ##### step 2: examine source for template details and determine destination project
-    builder.configure()
-
-    ##### step 3: create destination project if neccessary
+    ##### step 2: create destination project if neccessary
     builder.set_dest_project_and_path(sourceProject, currentProject, dest)
 
+    ##### step 3: examine source for template details and determine shared project
+    builder.configure()
+
     ##### step 4 create ensemble in destination project if needed
     if options.get("empty"):
         # don't create an ensemble
@@ -937,7 +979,7 @@ def clone(source, dest, ensemble_name=DefaultNames.EnsembleDirectory, **options)
     return builder.set_ensemble(isRemote, sourceProject, currentProject)
 
 
-def _create_local_config(clonedProject):
+def _create_local_config(clonedProject, logger):
     local_template = os.path.join(
         clonedProject.projectRoot, DefaultNames.LocalConfigTemplate
     )
@@ -962,10 +1004,12 @@ def _create_local_config(clonedProject):
             contents,
         )
 
-        logging.info(
+        logger.info(
             f'Generated new a local project configuration file at "{dest}"\n'
             "Please review it for any instructions on configuring this project."
         )
+        return True
+    return False
 
 
 def _get_unfurl_requirement_url(spec):
