@@ -271,6 +271,12 @@ class Project:
         self.workingDirs[localRepoPath] = RepoView(dict(name="", url=gitUrl), newRepo)
         return newRepo
 
+    def find_or_create_working_dir(self, repoURL, revision=None):
+        repo = self.find_git_repo(repoURL, revision)
+        if not repo:
+            repo = self.create_working_dir(repoURL, revision)
+        return repo
+
     def get_manifest_path(self, localEnv, manifestPath, can_be_empty):
         if manifestPath:
             # at this point a named manifest
@@ -687,6 +693,7 @@ class LocalEnv:
         parent=None,
         project=None,
         can_be_empty=False,
+        override_context=None,
     ):
         """
         If manifestPath is None find the first unfurl.yaml or ensemble.yaml
@@ -715,6 +722,9 @@ class LocalEnv:
             self.homeProject = self._get_home_project()
 
         self._resolve_path_and_project(manifestPath, can_be_empty)
+        if override_context:
+            # set after _resolve_path_and_project() is called
+            self.manifest_context_name = override_context
         if project:
             # this arg is used in init.py when creating a project
             # overrides what was set by _resolve_path_and_project()
@@ -826,7 +836,10 @@ class LocalEnv:
 
         if project:
             url, file, revision = split_git_url(project["url"])
-            repo = self.find_git_repo(url)
+            if currentProject:
+                repo = currentProject.find_git_repo(url)
+            if not repo:
+                repo = self.find_git_repo(url)
             if project.get("file"):
                 file = os.path.join(file, project.get("file"))
 
