@@ -443,7 +443,10 @@ class GitRepo(Repo):
         )
 
     def add_to_local_git_ignore(self, rule):
-        with open(os.path.join(self.repo.git_dir, "info", "exclude"), "a") as f:
+        path = os.path.join(self.repo.git_dir, "info")
+        if not os.path.exists(path):
+            os.makedirs(path)
+        with open(os.path.join(path, "exclude"), "a") as f:
             f.write("\n" + rule + "\n")
 
     def show(self, path, commitId):
