@@ -1,7 +1,7 @@
 TOSCA_VERSION = "tosca_simple_unfurl_1_0_0"
 
 # loaded by toscaparser extention manager using entrypoint declared in setup.cfg
-class plugindef_1_0_0(object):
+class plugindef_1_0_0:
     VERSION = TOSCA_VERSION
 
     DEFS_FILE = "tosca-ext.yaml"
