@@ -99,6 +99,7 @@ class UnfurlValidationError(UnfurlError):
 
 class UnfurlTaskError(UnfurlError):
     def __init__(self, task, message, log=logging.ERROR):
+        message = f"{task.changeId} on {task.target.name} {task.name}: {message}"
         super().__init__(message, True, False)
         self.task = task
         task._errors.append(self)
@@ -241,8 +242,10 @@ def load_class(klass, defaultModule="__main__"):
 
 _shortNameRegistry = {}
 
+
 def check_class_registry(kind):
-    return kind in _ClassRegistry or  kind in _shortNameRegistry
+    return kind in _ClassRegistry or kind in _shortNameRegistry
+
 
 def lookup_class(kind, apiVersion=None, default=None):
     if kind in _ClassRegistry:
@@ -273,13 +276,26 @@ def to_enum(enum, value, default=None):
         return value
 
 
+def to_dotenv(env: dict):
+    # https://hexdocs.pm/dotenvy/dotenv-file-format.html
+    keys = []
+    for k, v in env.items():
+        # k should be [a-zA-Z_]+[a-zA-Z0-9_]*
+        value = str(v)
+        if value.isalnum():
+            # always single quote to prevent interpolation
+            keys.append(f"{k}='{value}'")
+        else:
+            keys.append(f"{k}={repr(value)}")
+    return '\n'.join(keys)
+
 def dump(obj, tp, suffix="", yaml=None, encoding=None):
     from .yamlloader import yaml as _yaml
 
     try:
         if six.PY3:
             textEncoding = (
-                "utf-8" if encoding in [None, "yaml", "vault", "json"] else encoding
+                "utf-8" if encoding in [None, "yaml", "vault", "json", "env"] else encoding
             )
             f = io.TextIOWrapper(tp, textEncoding)
         else:
@@ -304,6 +320,10 @@ def dump(obj, tp, suffix="", yaml=None, encoding=None):
                 tp.write(obj)
                 return
 
+        if suffix.endswith(".env") or encoding == "env" and isinstance(obj, dict):
+            tp.write(codecs.encode(to_dotenv(obj), "utf-8"))
+            return
+
         # try to dump any other object as json
         if obj is not None:  # treat None as 0 byte file
             json.dump(obj, f, indent=2, sort_keys=True)
