@@ -224,7 +224,11 @@ class ReadOnlyManifest(Manifest):
         self.context = manifest.get("environment", CommentedMap())
         if localEnv:
             self.context = localEnv.get_context(self.context)
-        spec["inputs"] = self.context.get("inputs", spec.get("inputs", {}))
+        inputs = spec.get("inputs") or {}
+        context_inputs = self.context.get("inputs")
+        if context_inputs:
+            inputs.update(context_inputs)
+        spec["inputs"] = inputs
         self.update_repositories(
             manifest, relabel_dict(self.context, localEnv, "repositories")
         )
@@ -308,12 +312,20 @@ class YamlManifest(ReadOnlyManifest):
         if self.manifest.path:
             self.lockfilepath = self.manifest.path + ".lock"
 
-        more_spec = self._load_context(self.context, localEnv)
-        self._set_spec(manifest, more_spec, skip_validation)
-        assert self.tosca
         spec = manifest.get("spec", {})
-        status = manifest.get("status", {})
+        more_spec = self._load_context(self.context, localEnv)
+        # if we are exporting (thus skip_validation is True)
+        if skip_validation and spec.get("resource_templates"):
+            node_templates = more_spec["topology_template"]["node_templates"]
+            self._load_resource_templates(spec["resource_templates"], node_templates)
+        if self.context.get("instances"):
+            # add context instances to spec instances
+            self._load_resource_templates(self.context["instances"], spec.setdefault('instances', {}))
 
+        self._set_spec(spec, more_spec, skip_validation)
+        assert self.tosca
+
+        status = manifest.get("status", {})
         self.changeLogPath = manifest.get("jobsLog")
         self.jobsFolder = manifest.get("jobsFolder", "jobs")
         if not self.changeLogPath and localEnv and manifest.get("changes") is None:
@@ -425,6 +437,16 @@ class YamlManifest(ReadOnlyManifest):
             self.create_node_instance(key, val, root)
         return root
 
+    def _load_resource_templates(self, templates, node_templates):
+        # "resource_templates" are node templates that aren't included in the topology_template
+        # but are referenced by the deployment blueprints
+        # or are instances that are part of the environment
+        # XXX these might not be node_templates, need to check type
+        for name, tpl in templates.items():
+            # hacky way to exclude being part of the deployment plan and the manifest's status
+            tpl.setdefault("directives", []).append("virtual")
+            node_templates[name] = tpl
+
     def _load_context(self, context, localEnv):
         imports = context.get("imports")
         connections = relabel_dict(context, localEnv, "connections")
@@ -592,6 +614,8 @@ class YamlManifest(ReadOnlyManifest):
         return [{name: status}]
 
     def _save_entity_if_instantiated(self, resource, checkstatus=True):
+        if "virtual" in resource.template.directives:
+            return None
         if not resource.last_change and (
             not resource.local_status
             or (
